"""Init command for Ginpar projects.

This module implements the initialization command for the ginpar static content
generator.

`init` will prompt for a series of values to write the site configuration file.

Examples
--------

To initialize a project in a standard way to specify the configuration values::

    ginpar init

To skip the prompts and initialize the project with the default values::

    ginpar init --quick
    ginpar init --q

To force the initialization in case there is a directory with the same name
of the project to initialize::

    ginpar init --force
    ginpar init -f
"""

import os

import click
import yaml

from ginpar.utils.echo import info, echo, success, error, alert
from ginpar.utils.files import create_file, create_folder, try_remove
from ginpar.utils.strings import space_to_kebab


def prompt_site_config(quick):
    """Echo the prompts and create the configuration dict.

    Echo the instructions and configuration fields, store each input,
    and create a dictionary containing those values.

    Parameters
    ----------
    quick : bool
        Returns the default values immediatle if True.

    Returns
    -------
    dict
        Used to generate the site configuration file.
    """
    site = {
        "author": "David Omar",
        "sitename": "My site",
        "description": "This is a Ginpar project",
        "url": "/",
        "theme": "davidomarf/gart",
        "content_path": "sketches",
        "build_path": "public",
    }

    if quick:
        return site

    info("Welcome to ginpar! We'll ask for some values to initialize your project.")
    echo("")
    site["sitename"] = click.prompt("Site name", default=site["sitename"])
    site["description"] = click.prompt("Description", default=site["description"])
    site["author"] = click.prompt("Author", default=site["author"])
    site["url"] = click.prompt("url", default=site["url"])
    info("\nIf you're unsure about the next prompts, accept the defaults")
    echo("")
    site["theme"] = click.prompt("Theme", default=site["theme"])
    site["content_path"] = click.prompt("Sketches path", default=site["content_path"])
    site["build_path"] = click.prompt("Build path", default=site["build_path"])

    return site


def init(force, quick):
    """Main function of the module. This is what `ginpar init` calls.

    Parameters
    ----------
    force : bool
        Remove conflicting files when true.
    
    quick : bool
        Skip prompts when true.
    """

    if force:
        alert("You're forcing the initialization.")
        alert("This will replace any existent file relevant to the project.")
        click.confirm("Do you want to proceed?", abort=True)

    site = prompt_site_config(quick)
    path = space_to_kebab(site["sitename"]).lower()

    content_path = os.path.join(path, site["content_path"])
    config_yaml = os.path.join(path, "config.yaml")

    echo("\n---\n")
    
    if force:
        echo("\n---\n")
        try_remove(path)
        echo("\n---\n")

    create_folder(content_path)
    with open(config_yaml, "w") as file:
        yaml.dump(site, file)

    echo("\n---\n")
    success(
        "Done!\nRun `ginpar serve` or `ginpar build` and see your new site in action!\n"
    )



import os

from click.testing import CliRunner

from ginpar import cli


def test_quickstart():
    """Quickstart works properly on the default scenario.

    Default scenario is when the directory of the quickstart project doesn't
    exist in the current path: `./quickstart`.
    This also doesn't need to send the force flag.
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["quickstart"])
        assert result.exit_code == 0
        assert "The Ginpar sample site is ready." in result.output


def test_quickstart_existent_path():
    """Quickstart works properly when `./quickstart` exists.
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        os.mkdir("quickstart")
        file_path = os.path.join("quickstart", "a.txt")

        with open(file_path, "w+") as f:
            f.write("a")

        result = runner.invoke(cli, ["quickstart"])
        assert result.exit_code == 1


def test_quickstart_existent_path_force():
    """Quickstart works properly when `./quickstart` exists and --force
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        os.mkdir("quickstart")
        file_path = os.path.join("quickstart", "a.txt")

        with open(file_path, "w+") as f:
            f.write("a")

        result = runner.invoke(cli, ["quickstart", "--force"])
        assert result.exit_code == 0


def test_quickstart_removes_git_files():
    """Quickstart removes git files after cloning

    Check for the existence of any file that starts with `.git`.
    This will report the existent files.
    """
    import os

    runner = CliRunner()
    with runner.isolated_filesystem():
        git_files = []
        runner.invoke(cli, ["quickstart"])
        for _, d, f in os.walk("quickstart"):
            for name in d:
                if name.startswith(".git"):
                    git_files.append(name)
            for file in f:
                if file.startswith(".git"):
                    git_files.append(file)

        assert git_files == []



"""
    ginpar.utils
    ~~~~~~~~~~~~

    Implements the utilities used across modules in this package.
"""

import click


def echo(m):
    click.echo(m)


def info(m):
    click.secho(m, fg="blue")


def success(m):
    click.secho(m, fg="green")


def error(m):
    click.secho(m, fg="red")


def alert(m):
    click.secho(m, fg="yellow")



import os
import string
import json
import yaml

from jinja2 import Environment, FileSystemLoader


def dict_to_attrs(d):
    attrs = []
    for k, v in d.items():
        attrs.append(f'{k}="{v}"')
    attrs = " ".join(attrs)
    return attrs


delimiters = "/* ##ginpar */"

_INPUT_TEMPLATES_DIR = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "templates"
)

# Get a list that contains the name of the templates
_INPUT_TEMPLATES_LIST = list(
    map(
        lambda e: e.split(".")[0],
        filter(lambda e: e.endswith(".html"), os.listdir(_INPUT_TEMPLATES_DIR)),
    )
)

_jinja_env = Environment(
    loader=FileSystemLoader(_INPUT_TEMPLATES_DIR), trim_blocks=True
)

_jinja_env.filters["getattrs"] = dict_to_attrs


def makeValueGetter(attrs):
    _TEMPLATE = _jinja_env.get_template("retrieve.js")

    return _TEMPLATE.render(params=attrs)


def sketch_to_dict(s):
    """Receives the content of a sketch file with a JSON object
    inside a pair of ginpar delimiters"""

    ## Work only with the substring between two delimiters
    params_string = s.split(delimiters)[1]

    ## Remove the part of the string before the assignment
    params = params_string.split("=")[1]

    ## Remove the whitespace at the ends
    params = params.strip()

    ## Remove the ` characters (first and last elements)
    params = params[1:-1]

    return json.loads(params)


def to_kebab(s):
    s = s.lower().split(" ")
    return "-".join(s)


def input_tag(field):
    ## Obtain the html id
    id = to_kebab(field["name"])

    ##
    attrs = []
    for k, v in field["attrs"].items():
        attrs.append(f'{k}="{v}"')
    attrs = " ".join(attrs)

    if id in _INPUT_TEMPLATES_LIST:
        _input_template = _jinja_env.get_template(id + ".html")
    else:
        _input_template = _jinja_env.get_template("input.html")

    # print(attrs)

    return _input_template.render(id=id, name=field["name"], attrs=field["attrs"])


def form_tag(fields):
    form = ["<form>"]
    for f in fields:
        form.append(input_tag(f))
    form.append("\n</form>")
    form = "\n".join(form)
    return form


def add_name(fields):
    """Adds a `name` using the `var` when no `name` was specified"""
    for field in fields:
        if "name" not in field:
            field["name"] = " ".join(field["var"].split("_")).capitalize()
        field["id"] = to_kebab(field["name"])
    return fields


def sketch_index(sketch):
    return form_tag(sketch)



"""Quickstart command for Ginpar projects.

This module implements the quickstart command for the ginpar static content
generator.

`quickstart` will download the contents of the sample repository hosted at
`davidomarf/ginpar-quickstart <https://github.com/davidomarf/ginpar-quickstart>`_
into `./quickstart`.

This is aimed to provide an easier and faster way to start working in a Ginpar
project for people who isn't familiar with static content generators.

Example
-------

To create `./quickstart` and copy the contents of the sample repository::

    ginpar quickstart

If there's another directory named `quickstart`, you can force this command,
removing the existing directory::

    ginpar quickstart --force
    ginpar quickstart -f
"""
import os
import subprocess

import click

from ginpar.utils.echo import success, alert, echo, error
from ginpar.utils.files import try_remove
from ginpar.utils.git import clone_repo, delete_git_files

def quickstart(force):
    """Main function of the module. This is what `ginpar quickstart` calls.

    Parameters
    ----------
    force : bool
        Remove conflicting files when true.
    """

    repo = "davidomarf/ginpar-quickstart"
    path = os.path.abspath("quickstart")

    if force:
        alert("Forcing quickstart. This will replace existent directories and files.\n")
        try_remove("quickstart")
        echo("")

    if os.path.isdir(path):
        error(f"`{path}` already exists.")
        echo("Delete it manually or run `ginpar quickstart -f` to force")
        raise click.Abort()

    if clone_repo(repo, path) == 0:
        if delete_git_files(path) == 0:
            echo(f"\nThe Ginpar sample site is ready.\n")
            echo("Run `cd quickstart` to move to the project directory.")
            echo("Then run `ginpar build` or `ginpar serve` to see it working.")
    else:
        raise click.Abort()


from click.testing import CliRunner
from ginpar import cli


def test_init():
    runner = CliRunner()
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["init", "-q"])
        assert result.exit_code == 0
        assert "Done" in result.output



"""This module handles repository downloads
"""
import os
import subprocess

from ginpar.utils.echo import success, echo, error
from ginpar.utils.files import try_remove

def clone_repo(repo, path):
    """Clone the contents of a repository in a custom path

    Parameters
    ----------
    repo : str
        GitHub repository as in "{USER}/{REPO}"
    path : str
        Path to clone the repository to
    """

    repo_url = f"https://github.com/{repo}.git"

    echo(f"> Cloning {repo} in `{path}`")
    try:
        subprocess.call(["git", "clone", repo_url, path, "--quiet"])
    except OSError:
        error("You don't have git installed in your machine.")
        echo("Please install git and rerun or download the files manually:")
        echo(f"\t{repo_url}")
        return 1

    success(f"Successfully cloned {repo}.\n")
    return 0

def delete_git_files(path):
    """Delete the git files to only keep the relevant files

    Parameters
    ----------
    path : str
        Path to look for git files
    """

    echo("> Deleting .git files")
    try:
        print(os.path.join(path, ".git"))
        try_remove(os.path.join(path, ".git"))
        success("Successfully deleted .git files")
    except:
        error(f"Couldn't delete files. Delete all .git files manually in `{path}`")
        return 1

    return 0



import shutil
import os

from ginpar.utils.echo import echo, error, success, info


def check_existence(path):
    return os.path.isfile(path) or os.path.isdir(path)


def create_file(file, content):
    echo(f"> Creating `{file}`:")
    try:
        config = open(file, "r")
    except:
        try:
            config = open(file, "w+")
        except:
            error("Failure.")
        else:
            config.write(content)
            config.close()
            success("Success.")
    else:
        config.close()
        error("Failure. It already exists.")


def create_folder(folder, force=False):
    echo(f"> Creating `{folder}`:")
    try:
        os.makedirs(folder)
    except FileExistsError:
        error("Failure. It already exists.")
    except:
        error("Failure.")
    else:
        success("Sucess")


def copy_folder(fr, to, force=False):
    echo(f"\n> Copying `{to}` from `{fr}`:")

    exists = check_existence(to)

    if exists and force:
        try_remove(to)

    try:
        shutil.copytree(fr, to)
    except FileExistsError:
        error(f"Failure. It already exists.")
    else:
        success(f"Success.")


def try_remove(path):
    print(path)
    if os.path.isdir(path):
        echo(f"> `{path}` already exists. Attemping removal:")
        try:
            shutil.rmtree(path)
        except:
            error("Failure. Restart or delete manually.")
        else:
            success("Success.")
    elif os.path.isfile(path):
        echo(f"> `{path}` already exists. Attemping removal:")
        try:
            os.remove(path)
        except:
            error(f"Failure. Restart or delete manually.")
        else:
            success(f"Success.")
    else:
        echo(f"> `{path}` doesn't exist.")
        info("Skipping.")



import os

from click.testing import CliRunner
from ginpar import cli

def test_new():
    """New works properly on the default scenario.

    Default scenario is when the directory of the new sketch doesn't
    exist in the source_path..
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        ## Initialize a new project and cd it
        runner.invoke(cli, ["init", "-q"])
        os.chdir("my-site")

        result = runner.invoke(cli, ["new", "test"])
        
        data_path = os.path.join("sketches", "test", "data.yaml")
        sketch_path = os.path.join("sketches", "test", "sketch.js")
        assert result.exit_code == 0
        assert os.path.isfile(data_path)
        assert os.path.isfile(sketch_path)
       

def test_new_existent():
    """New works properly when the sketch already exists
    """
    runner = CliRunner()
    with runner.isolated_filesystem():
        ## Initialize a new project and cd it
        runner.invoke(cli, ["init", "-q"])
        os.chdir("my-site")
        
        ## Create test
        runner.invoke(cli, ["new", "test"])

        result = runner.invoke(cli, ["new", "test"])
        assert result.exit_code == 1
       


"""New sketch creation command for Ginpar projects.

This module implements the sketch creation command for the ginpar static content
generator.

`new` will read the configuration file in search for `source_path` and
create a new directory in there with the specified name, which by default is
`new-sketch-{n}`.

This directory will contain the two required files with a boilerplate code.

Examples
--------

To create a new sketch `rectangle`::

    ginpar new rectangle

To start a new sketch with the default name `new-sketch-{n}`::

    ginpar new
"""
import os
import yaml

import click
from jinja2 import Environment, FileSystemLoader

from ginpar.utils.files import create_folder, create_file
from ginpar.utils.echo import echo, error, success

## TODO: Move read_config into a shared library inside utils
def read_config(path):
    """Create a dictionary out of the YAML file received

    Paremeters
    ----------
    path : str
        Path of the YAML file.
    """
    with open(path, "r") as stream:
        try:
            config = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
    return config


def new(sketch):
    """Main function of the module. This is what `ginpar new` calls.

    Parameters
    ----------
    sketch : str
        Name of the sketch to create
    """

    _SITE = "config.yaml"
    site = read_config(_SITE)

    path = os.path.join(site["content_path"], sketch)

    _TEMPLATES_DIR = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "templates", "sketch"
    )
    _jinja_env = Environment(loader=FileSystemLoader(_TEMPLATES_DIR), trim_blocks=True)

    if os.path.isdir(path):
        error(f"Failure.")
        echo(f"{path} already exists.")
        raise click.Abort()
    
    create_folder(path)

    sketch_template = _jinja_env.get_template("sketch.js")
    data_template = _jinja_env.get_template("data.yaml")

    create_file(os.path.join(path, "sketch.js"), sketch_template.render())
    create_file(os.path.join(path, "data.yaml"), data_template.render())

    echo(f"\nYour new sketch {path} is ready.\n")


"""Definition of the CLI commands for Ginpar.

This module defines the different commands available for the ginpar static
content generator.

Examples
--------

To get the list of available commands and options run::

    ginpar
"""

import click


@click.group()
@click.version_option(message="%(prog)s v%(version)s")
def cli():
    """
    Ginpar is a static content generator for interactive P5.js sketches,
    awkwardly named after Generative Interactive Parametrisable Canvases.
    """
    pass


@cli.command()
@click.option(
    "--path",
    "-p",
    default="public",
    type=click.Path(),
    help=(
        "The PATH where the site will be built. [ <config.build_path>, public ] "
        "This path is relative to the current directory. When no option is provided "
        "Ginpar will read the <config.build_path> from the configuration file."
        ),
)
def build(path):
    """Build the project content into PATH.

        `ginpar build` will read your configuration file, fetch all the sketches inside
        your <config.content_path>, and build your static site inside PATH, which
        defaults to <config.build_path>, or public if it doesn't exist.

        This operation will wipe all the content from PATH in each run, so you must not
        make modifications you expect to preserve.
    """
    from ginpar.build import build as ginpar_build

    click.echo("")
    ginpar_build(path)
    click.echo("")


@cli.command()
@click.option(
    "--force",
    "-f",
    default=False,
    is_flag=True,
    help=(
        "If Ginpar finds an existing directory with the same name of the "
        "project being initialized, it'll force its removal. "
        "Only do this if you're completely sure you want to do it."
        ),
)
@click.option(
    "--quick",
    "-q",
    default=False,
    is_flag=True,
    help=(
        "Skip the prompts and use the default values for the configuration file. "
        "You can still modify the variables later by manually updating your "
        "configuration file."
    ),
)
def init(force, quick):
    """Initialize a new project in PATH.

        `ginpar init` will prompt you for a series of values that will be used to
        generate the configuration and file structure of your project.
    """
    from ginpar.init import init as ginpar_init

    click.echo("")
    ginpar_init(force, quick)
    click.echo("")


@cli.command()
@click.argument("sketch")
def new(sketch):
    """Create a new SKETCH.

        `ginpar new` will create a new sketch structure inside your <config.content_path>.

        You must specify the name of the sketch.

        If there's an existing sketch with the same name, it'll throw an error and ask
        for a different name.
    """
    from ginpar.new import new as ginpar_new

    click.echo("")
    ginpar_new(sketch)
    click.echo("")


@cli.command()
@click.option(
    "--force",
    "-f",
    default=False,
    is_flag=True,
    help=(
        "If Ginpar finds an existing directory with the same name of the sample content, "
        "it'll force its removal. "
        "Only do this if you're completely sure you want to do it."
        ),
)
def quickstart(force):
    """Import a working sample project.
    
        `ginpar quickstart` will download the contents of the sample project, hosted
        at github: davidomarf/ginpar-quickstart in the current directory.
    """
    from ginpar.quickstart import quickstart as ginpar_quickstart

    click.echo("")
    ginpar_quickstart(force)
    click.echo("")


@cli.command()
@click.option("--port", "-p", default=8080, help="Port of the server")
def serve(port):
    """Start a new server in localhost:PORT.

        `ginpar serve` will trigger `ginpar build`, and start a new server inside
        <config.build_path>.

        Every time you modify a file that is part of the project's source code the
        site gets built again.
    """
    from ginpar.serve import serve as ginpar_serve

    click.echo("")
    ginpar_serve(port)
    click.echo("")


if __name__ == "__main__":
    cli()



import os

from click.testing import CliRunner
from ginpar import cli


def test_init():
    runner = CliRunner()
    with runner.isolated_filesystem():
        # Create a project with default values
        runner.invoke(cli, ["init", "-q"])
        # => Creates a path "my-site/"

        # Change directories
        os.chdir("my-site")

        result = runner.invoke(cli, ["build"])
        assert result.exit_code == 0



"""Build command for Ginpar projects.

This module implements the building command for the ginpar static content
generator.

`build` will read the configuration file in search for `build_path` and
`source_path`. If not defined, `build` will use `"public"` and
`"sketches`, respectively.

Examples
--------

To build your project according to your specifications in `config.yaml`::

    ginpar build

To build targeting a custom path `_site/`::

    ginpar build --path="_site"

Notes
-----

You cannot specify the content path. It is either ``config.content_path`` or
``"sketches"``.
"""

import os
import shutil

import yaml
import click
from jinja2 import Environment, FileSystemLoader

from ginpar.utils.echo import echo, success
from ginpar.utils.strings import unkebab, camel_to_space, space_to_kebab
from ginpar.utils.git import clone_repo, delete_git_files

import ginpar.generators as gg

import click


def add_name_key(e):
    """Adds the name field for the parameter received

    The received dict has the format `{ NAME: { "attrs": attrs, ... }}`,
    where NAME is the only key that the dict should have.

    If e[NAME]["name"] exists, it will return the same element. Otherwise, it
    will create a ["name"] key using NAME and filtering to make it space case.

    Examples
    --------

    >>> e = {"MY_TEST": {"attrs": {} }}
    >>> add_name_key(e)
    {"MY_TEST": {"attrs": {}, "name": "My test"}}

    >>> a = {"SECOND_TEST": {"attrs": {}, "name": "OwO" }}
    >>> add_name_key(e)
    {"SECOND_TEST": {"attrs": {}, "name": "OwO" }}

    Parameters
    ----------
    e : dict
        Dictionary with only one key `k` that will be used to set e["name"]
    """
    name = list(e)[0]

    if "name" in e[name]:
        return e
    else:
        e[name]["name"] = camel_to_space(name).capitalize()
        return e


def add_id_key(e):
    """Adds the id field for the parameter received

    The received dict has the format `{ NAME: { "attrs": attrs, "name": NAME }}`.

    This will createa e[NAME]["id"] using e[NAME]["name] and filtering it to
    create a kebab-case.

    Examples
    --------

    >>> e = {"SECOND_TEST": {"attrs": {}, "name": "Burger" }}
    >>> add_name_key(e)
    {"SECOND_TEST": {"attrs": {}, "name": "Burger" }}

    Parameters
    ----------
    e : dict
        Dictionary with a key [NAME]["name"]
    """
    name = list(e)[0]
    e[name]["id"] = space_to_kebab(e[name]["name"]).lower()
    return e


def get_sketches(content_path):
    """Obtain the list of **valid** sketches inside `path`.

    Valid sketches are directories containing at least two files:
    `sketch.js` and `data.yaml`.

    This function will create a list of sketch objects containing
    `name`, `script`, and `data`.

    Parameters
    ----------
    content_path : str
        The path containing the sketches to fetch.

    Returns
    -------
    list
        Individual elements contain `{"name", "script", "data"}`.
    """

    sketches = []

    # Create a list with all the directories inside path
    for r, d, _ in os.walk(content_path):
        for sketch in d:
            sketches.append(
                {
                    "name": sketch,
                    "script": os.path.join(r, sketch, "sketch.js"),
                    "data": os.path.join(r, sketch, "data.yaml"),
                }
            )

    return map(
        # Convert the data.yaml file into a dictionary
        convert_information,
        # Remove sketch if either `sketch.js` or `data.yaml` don't exist
        filter(
            lambda a: os.path.isfile(a["script"]) and os.path.isfile(a["data"]),
            sketches,
        ),
    )


def convert_information(sketch):
    """Convert the `["data"]` field of a sketch into a Python dictionary.

    Parameters
    ----------
    sketch : dict
        It contains the sketch information. Must contain `["data"]`, and
        `["data"]` must be a YAML-valid string.


    Returns
    -------
    dictionary
        `sketch` but with the updated `["data"]` field.
    """

    path = sketch["data"]
    with open(path, "r") as stream:
        try:
            parsed_data = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
    sketch["data"] = parsed_data
    return sketch


def create_publishing_directory(build_path):
    """Remove existing directories with the same name, and create again.

    Parameters
    ----------
    build_path : str
        Path of the build.
    """

    if os.path.exists(build_path):
        shutil.rmtree(build_path)
    os.mkdir(build_path)


def copy_theme(build_path, theme):
    """Copy the theme static content into the build static directory.

    Parameters
    ----------
    build_path : str
        Path of the build.

    theme : str
        Name of the theme to install.
    """

    ## Copy the static/ folder of the theme
    shutil.copytree(
        os.path.join("themes", theme, "static"), os.path.join(build_path, "static")
    )


def render_index(build_path, sketches, site, page_template):
    """Render the index using the list of sketches and site configuration

    The index is rendered using a Jinja2 template inside the theme `templates/`
    directory.

    The index template must receive `sketches`, containing a list of the sketches
    names; and `site`, containing the site configuration from ``config.yaml``.

    Parameters
    ----------
    build_path : str
        Path of the build.

    sketches : list
        Contains all the sketches in the project. Must contain at leas `["name"]`.
    
    site : dict
        Contains the site information, such as `sitename` and `author`.

    page_template : Jinja2.Template
        Jinja2 template to render the sketch.
    """

    # Open the index in the build path for writing
    index = open(os.path.join(build_path, "index.html"), "w")

    # Write the contents of the rendered template into the index file
    index.write(
        page_template.render(sketches=map(lambda a: a["name"], sketches), site=site)
    )
    index.close()

def param_to_dict(param):
    param_var = list(param)[0]
    return {
        "var": param_var,
        "id": param_var.lower(),
        "attrs": param[param_var]["attrs"],
        "name": param_var.capitalize()
    }


def render_sketch_page(build_path, sketch, site, page_template, input_templates):
    """Render a sketch page

    This generates the page for a single sketch. This will convert the
    `sketch["data"]` into a form that will control the variables of the
    script.

    When `sketch["data"]` doesn't define fields that may be used at the moment
    of the form generation, Ginpar will instead look up for those fields in
    `site["sketch_defaults"]`. 

    When both `sketch["data"]` and `site["sketch_defaults"]` don't define those
    fields, Ginpar will use the default values. 

    `Ginpar default values for sketch data <https://ginpar.readthedocs.io/en/latest/data.html>`_

    Parameters
    ----------

    build_path : str
        Path of the build.

    sketch : dict
        Sketch information. Must contain `["data"]` and `["name"]`

    site : dict
        Site configuration.
        
    page_template : Jinja2.Template
        Jinja2 template to render the sketch.
    """
    ## Create a directory with the sketch title
    os.mkdir(os.path.join(build_path, sketch["name"]))
    params = list(map(param_to_dict, sketch["data"]["params"]))
    default_input = next((x for x in input_templates if x.name.endswith("default.html")))

    content = ""
    for param in params:
        input_type = param["attrs"]["type"]
        template = next(
            (x for x in input_templates if x.name.endswith(input_type + ".html")),
            default_input)
        content = content + template.render(param = param)

    ## Create index.html
    sketch_index = open(f"public/{sketch['name']}/index.html", "w+")
    sketch_index.write(
        page_template.render(
            sketch=unkebab(sketch["name"]), form = "<form>\n" + content + "\n</form>", site=site
        )
    )
    sketch_index.close()

    ## Create sketch.js
    sketch_path = f"public/{sketch['name']}/sketch.js"
    sketch_script = open(sketch_path, "w+")

    ## Copy all the content from original sketches/{title}.js to sketch.js
    sf = open(sketch["script"], "r")

    sketch_script.write(gg.makeValueGetter(list(params)))
    for x in sf.readlines():
        sketch_script.write(x)
    sf.close()
    sketch_script.close()


def read_config(path):
    """Create a dictionary out of the YAML file received

    Paremeters
    ----------
    path : str
        Path of the YAML file.
    """
    with open(path, "r") as stream:
        try:
            config = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
    return config


def dict_to_attrs(d):
    attrs = []
    for k, v in d.items():
        attrs.append(f'{k}="{v}"')
    attrs = " ".join(attrs)
    return attrs


def build(path):
    """Main function of the module. This is what `ginpar build` calls.

    Parameters
    ----------
    build_path : str
        Path of the build.
    """

    _SITE_FILE = "config.yaml"
    _SITE = read_config(_SITE_FILE)

    _THEME = _SITE["theme"].split("/")[1]
    _THEME_PATH = os.path.join("themes", _THEME)
    
    _TEMPLATES_PATH = os.path.join(_THEME_PATH, "templates")
    _SKETCHES_PATH = _SITE["content_path"]

    _jinja_env = Environment(loader=FileSystemLoader(_TEMPLATES_PATH), trim_blocks=True)

    _jinja_env.filters["unkebab"] = unkebab
    _jinja_env.filters["getattrs"] = dict_to_attrs

    input_templates = list(map(
        lambda t : _jinja_env.get_template(t),
        filter(
            lambda t : t.startswith(os.path.join("form", "inputs")),
            _jinja_env.list_templates()
        )
    ))

    if not os.path.isdir(_THEME_PATH):
        clone_repo(_SITE["theme"], _THEME_PATH)
        delete_git_files(_THEME_PATH)

    create_publishing_directory(path)
    echo(f"Building in `{os.path.abspath(path)}`")

    copy_theme(path, _THEME)
    echo(f"Building using theme `{_THEME}`")

    ## Create the sketches list
    sketches = list(get_sketches(_SKETCHES_PATH))
    echo(f"Found {len(sketches)} sketch(es)")
    sketches.sort(key=lambda a : a["data"]["date"], reverse=True)

    render_index(path, sketches, _SITE, _jinja_env.get_template("index.html"))
    echo("Building main page")

    echo("Building sketches:")
    for sketch in sketches:
        echo(f"  Building {sketch['name']}")
        render_sketch_page(path, sketch, _SITE, _jinja_env.get_template("sketch.html"), input_templates)

    success("Success.")



"""Serve command for Ginpar projects.

This module implements the server starting command for the ginpar static content
generator.

`serve` will start a live-reloading server in a specified port.

Examples
--------

To start a new server in the default port `8080`::

    ginpar serve

To start a new server in a custom port::

    ginpar serve --port=3000
    ginpar serve -p=3000
"""
import click
import yaml
from livereload import Server, shell


## TODO: Move read_config into a shared library inside utils
def read_config(path):
    """Create a dictionary out of the YAML file received

    Paremeters
    ----------
    path : str
        Path of the YAML file.
    """
    with open(path, "r") as stream:
        try:
            config = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)
    return config


def serve(port):
    """Main function of the module. This is what `ginpar serve` calls.

    Parameters
    ----------
    port : int
        The port of the server
    """
    site = read_config("config.yaml")

    server = Server()

    server.watch(site["content_path"], 'ginpar build')
    server.serve(port=port, root=site["build_path"])


import setuptools

version = '0.7.1'

requires = ['jinja2 >= 2.7', 'pyyaml', 'click']

setup_requires = ['wheel']

entry_points = {
    'console_scripts': [
        'ginpar = ginpar.cli:cli',
    ]
}

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='ginpar',
    version=version,
    url='https://github.com/davidomarf/ginpar',
    author='David Omar Flores Chávez',
    author_email='david@davidomar.com',
    description='A static content generator for interactive and parametrisable p5.js canvases.',
    license='MIT',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=requires,
    entry_points=entry_points,
    classifiers=[
         'Programming Language :: Python :: 3',
         'License :: OSI Approved :: MIT License',
         'Operating System :: OS Independent',
    ],
)